@@ -86,9 +86,16 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def ls_remove_dns_record(self, switch_uuid, dns_uuid):
+    def ls_remove_dns_record(self, switch_uuid, dns_uuid, if_exists=False):
         """Remove the 'dns_record' from the switch's 'dns_records' list
 
+        :param switch_uuid: The uuid of the switch
+        :type switch_uuid:  string or uuid.UUID
+        :param dns_uuid:    The uuid of the DNS record
+        :type dns_uuid:     string or uuid.UUID
+        :param if_exists:   If True, don't fail if the DNS record
+                            doesn't exist
+        :type if_exists:    boolean
         :returns: :class:`Command` with RowView result
         """
 
@@ -140,6 +147,111 @@ class API(api.API):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def pg_acl_add(self, port_group, direction, priority, match, action,
+                   log=False):
+        """Add an ACL to 'port_group'
+
+        :param port_group: The name or uuid of the port group
+        :type port_group:  string or uuid.UUID
+        :param direction:  The traffic direction to match
+        :type direction:   'from-lport' or 'to-lport'
+        :param priority:   The priority field of the ACL
+        :type priority:    int
+        :param match:      The match rule
+        :type match:       string
+        :param action:     The action to take upon match
+        :type action:      'allow', 'allow-related', 'drop', or 'reject'
+        :param log:        If True, enable packet logging for the ACL
+        :type log:         boolean
+        :returns:          :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def pg_acl_del(self, port_group, direction=None, priority=None,
+                   match=None):
+        """Remove ACLs from 'port_group'
+
+        If only port_group is supplied, all the ACLs from the logical switch
+        are deleted. If direction is also specified, then all the flows in
+        that direction will be deleted from the Port Group. If all the fields
+        are given, then only flows that match all fields will be deleted.
+
+        :param port_group: The name or uuid of the port group
+        :type port_group:  string or uuid.UUID
+        :param direction:  The traffic direction to match
+        :type direction:   'from-lport' or 'to-lport'
+        :param priority:   The priority field of the ACL
+        :type priority:    int
+        :param match:      The match rule
+        :type match:       string
+        :returns:          :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def pg_acl_list(self, port_group):
+        """Get the ACLs for 'port group'
+
+        :param port_group: The name or uuid of the switch
+        :type port_group:  string or uuid.UUID
+        :returns:          :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def qos_add(self, switch, direction, priority, match, rate=None,
+                burst=None, dscp=None, may_exist=False, **columns):
+        """Add an Qos rules to 'switch'
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param direction: The traffic direction to match
+        :type direction:  'from-lport' or 'to-lport'
+        :param priority:  The priority field of the QoS
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :param dscp:      The dscp mark to take upon match
+        :type dscp:       int
+        :param rate:      The rate limit to take upon match
+        :type rate:       int
+        :param burst:     The burst rate limit to take upon match
+        :type burst:      int
+        :param may_exist: If True, don't fail if the QoS rule already exists
+        :type may_exist:  boolean
+        :param columns:   Additional columns to directly set on the switch
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def qos_del(self, switch, direction=None, priority=None, match=None):
+        """Remove Qos rules from 'switch'
+
+        If only switch is supplied, all the QoS rules from the logical switch
+        are deleted. If direction is also specified, then all the flows in
+        that direction will be deleted from the logical switch. If all the
+        fields are given, then only flows that match all fields will be
+        deleted.
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param direction: The traffic direction to match
+        :type direction:  'from-lport' or 'to-lport'
+        :param priority:  The priority field of the QoS
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def qos_list(self, switch):
+        """Get the Qos rules for 'switch'
+
+        :param switch: The name or uuid of the switch
+        :type switch:  string or uuid.UUID
+        :returns:      :class:`Command` with RowView list result
+        """
+
     @abc.abstractmethod
     def lsp_add(self, switch, port, parent_name=None, tag=None,
                 may_exist=False, **columns):
@@ -430,6 +542,26 @@ class API(api.API):
         :returns:
         """
 
+    @abc.abstractmethod
+    def lrp_set_options(self, port, **options):
+        """Set options related to the type of 'port'
+
+        :param port:    The name or uuid of the port
+        :type port:     string or uuid.UUID
+        :param options: keys and values for the port 'options' dict
+        :type options:  key: string, value: string
+        :returns:       :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_get_options(self, port):
+        """Get the type-specific options for 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :returns:    :class:`Command` with dict result
+        """
+
     @abc.abstractmethod
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
@@ -554,11 +686,15 @@ class API(api.API):
     def lb_del(self, lb, vip=None, if_exists=False):
         """Remove a load balancer or just the VIP from a load balancer
 
+        If all vips of the load balancer are removed, the load balancer
+        instance will be removed as well. If vip parameter is 'None'
+        this will cause all vips (and load balancer) to be removed.
+
         :param lb:        The name or uuid of a load balancer
         :type lb:         string or uuid.UUID
-        :param vip:       The VIP on the load balancer to match
+        :param vip:       The VIP to match. If None, match all vips
         :type:            string
-        :param if_exists: If True, don't fail if the port doesn't exist
+        :param if_exists: If True, don't fail if the lb/vip doesn't exist
         :type if_exists:  boolean
         """
 
@@ -747,12 +883,15 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def dns_remove_record(self, uuid, hostname):
+    def dns_remove_record(self, uuid, hostname, if_exists=False):
         """Remove the 'hostname' from the 'records' field of the DNS row
 
         :param uuid: The uuid of the DNS row to set the records with
         :type uuid:  string or uuid.UUID
         :param hostname: hostname as the key to the record dict
+        :param if_exists:   If True, don't fail if the DNS record
+                            doesn't exist
+        :type if_exists:    boolean
         :returns:    :class:`Command` with no result
         """
 
@@ -768,10 +907,10 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def pg_add(self, name, may_exist=False, **columns):
+    def pg_add(self, name=None, may_exist=False, **columns):
         """Create a port group
 
-        :param name:        The name of the port group
+        :param name:        The name of the port group (optional)
         :type name:         string
         :param may_exist:   If True, don't fail if the port group already
                             exists
@@ -788,7 +927,8 @@ class API(api.API):
 
         :param name:        The name of the port group
         :type name:         string
-        :param if_exists:   If True, don't fail if the router doesn't exist
+        :param if_exists:   If True, don't fail if the port group
+                            doesn't exist
         :type if_exists:    boolean
         :returns:           :class:`Command` with no result
         """
@@ -799,9 +939,9 @@ class API(api.API):
 
         :param pg_id:     The name or uuid of the port group
         :type pg_id:      string or uuid.UUID
-        :param lsp:       A list of :class:`Command` with a Logical_Switch_Port
+        :param lsp:       A list of :class:`Command` Logical_Switch_Port
                           instance result or UUID
-        :type lsp:        A list of :class:`Command` with a Logical_Switch_Port
+        :type lsp:        A list of :class:`Command` Logical_Switch_Port
                           or string or uuid.UUID
         A Logical_Switch_Port instance or string
                           or uuid.UUID
@@ -814,9 +954,9 @@ class API(api.API):
 
         :param pg_id:     The name or uuid of the port group
         :type pg_id:      string or uuid.UUID
-        :param lsp:       A list of :class:`Command` with a Logical_Switch_Port
+        :param lsp:       A list of :class:`Command` Logical_Switch_Port
                           instance result or UUID
-        :type lsp:        A list of :class:`Command` with a Logical_Switch_Port
+        :type lsp:        A list of :class:`Command` Logical_Switch_Port
                           or string or uuid.UUID
         :type if_exists:  If True, don't fail if the logical port(s) doesn't
                           exist
@@ -825,30 +965,77 @@ class API(api.API):
         """
 
     @abc.abstractmethod
-    def pg_add_acls(self, pg_id, acl):
-        """Add a list of ACL to a port group
+    def pg_get(self, pg_id):
+        """Get port group
 
-        :param pg_id:     The name or uuid of the port group
-        :type pg_id:      string or uuid.UUID
-        :param acl:       The ACL instance or UUID
-        :type acl:        A list of ACL instance or string or uuid.UUID
-        :param acl:       A list of :class:`Command` with an ACL instance
-                          result or UUID
-        :type acl:        A :class:`Command` with an ACL or string
-                          r uuid.UUID
-        :returns:         :class:`Command` with no result
+        :param pg_id: The name or uuid of the port group
+        :type pg_id:  string or uuid.UUID
+        :returns:     :class:`Command` with RowView result
         """
 
     @abc.abstractmethod
-    def pg_del_acls(self, pg_id, acl, if_exists=False):
-        """Delete a list of ACL from a port group
+    def ha_chassis_group_add(self, name, may_exist=False, **columns):
+        """Create a HA Chassis Group
 
-        :param pg_id:     The name or uuid of the port group
-        :type pg_id:      string or uuid.UUID
-        :type acl:        A list of ACL instance or string or uuid.UUID
-        :param acl:       A list of :class:`Command` with an ACL instance
-                          result or UUID
-        :type if_exists:  If True, don't fail if the ACL(s) doesn't exist
-        :type if_exists:  boolean
-        :returns:         :class:`Command` with no result
+        :param name:        The name of the ha chassis group
+        :type name:         string
+        :param may_exist:   If True, don't fail if the ha chassis group
+                            already exists
+        :type may_exist:    bool
+        :param columns:     Additional columns to directly set on the ha
+                            chassis group (e.g external_ids)
+        :type columns:      dictionary
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ha_chassis_group_del(self, name, if_exists=False):
+        """Delete a HA Chassis Group
+
+        :param name:        The name of the ha chassis group
+        :type name:         string
+        :param if_exists:   If True, don't fail if the ha chassis group
+                            doesn't exist
+        :type if_exists:    boolean
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def ha_chassis_group_get(self, name):
+        """Get HA Chassis Group
+
+        :param name: The name or uuid of the ha chassis group
+        :type name:  string or uuid.UUID
+        :returns:    :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ha_chassis_group_add_chassis(self, hcg_id, chassis, priority,
+                                     **columns):
+        """Add a HA Chassis to a HA Chassis Group
+
+        :param hcg_id:   The name or uuid of the ha chassis group
+        :type hcg_id:    string or uuid.UUID
+        :param chassis:  The name of the ha chassis
+        :type chassis:   string
+        :param priority: The priority of the ha chassis
+        :type priority:  int
+        :param columns:  Additional columns to directly set on the ha
+                         chassis (e.g external_ids)
+        :type columns:   dictionary
+        :returns:        :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def ha_chassis_group_del_chassis(self, hcg_id, chassis, if_exists=False):
+        """Delete a HA Chassis from a HA Chassis Group
+
+        :param hcg_id:     The name or uuid of the ha chassis group
+        :type hcg_id:      string or uuid.UUID
+        :param chassis:    The name of the ha chassis
+        :type chassis:     string
+        :param if_exists:  If True, don't fail if the ha chassis
+                           doesn't exist
+        :type if_exists:   boolean
+        :returns:          :class:`Command` with no result
         """
